***** File description **************************************************
*** File:        mainLoop.s
***              (This is a part of Norbert Gb's PASSSAVE)
*** Platform:    HP48GX
*** Language:    System-RPL
*** Description: Input:
***                <2>  password-string
***                <1>  data (list of lists)
***              Output:
***                <3'> probably changed <2>
***                <2'> probably changed <1>
***                <1'> flag: true, if data should be saved.
***              
***              This is passSave-s main-loop. We let the user view
***              and edit the password and data.
***              
*** Author:      Norbert Gb (goeb@mathematik.uni-kl.de)
*** Date:        02/2000
***
*** License/Disclaimer **************************************************
*** This program is free software; you can redistribute it and/or  modify
*** it under the terms of the GNU General Public License as published  by
*** the Free Software Foundation; either version 2, or (at  your  option)
*** any later version.
*** 
*** This program is distributed in the hope that it will be  useful,  but
*** WITHOUT  ANY  WARRANTY;  without  even  the   implied   warranty   of
*** MERCHANTABILITY or FITNESS FOR A  PARTICULAR  PURPOSE.  See  the  GNU
*** General Public License for more details.
*** 
*** You should have received a copy of the  GNU  General  Public  License
*** along  with  this  program;  if  not,  write  to  the  Free  Software
*** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
*** USA.
*************************************************************************

*  ( hxs  --> str )
ASSEMBLE
=REMOVEN{} EQU #0E4DE
        NIBASC  /HPHP48-D/
RPL
:: 
    CK2NOLASTWD 
    CK&DISPATCH1 #00035 
    :: 
       ( stack: [passwd-str]-[data-list] )
       
       ( local vars: {1} first displayed entry, 1..? )
       (             {2} offset for selected entry, 1..7 - the selected entry )
       (                 has position {1}+{2}-1 in the data-list and position )
       (                 {2} in the sublist starting at pos {1} of the data. )
       ONEONE
       { NULLLAM NULLLAM } 
       BIND
         
	 ( prepare the display and show the menu-line: )
	 ClrDA1IsStat
	 RECLAIMDISP
	 ID stdMenu
 
         ( stack: [passwd-str]-[data-list] )
	 1GETLAM
	 ID extractHeaders
	 ( The selected position ought to be 1, iff the data-list is not )
	 ( empty, 0 if it is: )
	 OVER
	 LENCOMP
         2GETLAM
	 #MIN
	 DUP
	 2PUTLAM
	 
         ( start the key-event-loop )
         BEGIN
           ( stack: [passwd-str]-[data-list]-[win-header list]-[sel pos] )
	   ( display the header-strings )
	   ID dispStrings
	 
	   ( get some key, ignore modifiers )
	   WaitForKey
	   DROP

           ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	   (        ...-[selected pos]-[key num] )
	   
	   ::
             DUP ONE #= case 
  	       ::
	         ( key[A]: add )
		 DROP ( we do not need the keycode any longer )
		 2PUTLAM ( store the selection )
		 DROP ( drop the window )
                 ( stack: [passwd-str]-[data-list] )
		 
		 ( create a list of six empty strings )
		 NULL$
		 SIX
		 NDUPN
		 #1+
		 #1-{}N 
		 ( edit the strings in the list )
		 ID editEntry

		 ( restore the menu-line )
	         ID stdMenu
		 
		 ( append the new entry to the data-list )
		 >TCOMP

		 ( recalculate the window-headers )
		 1GETLAM
		 ID extractHeaders
		 2GETLAM

		 ( if there was nothing selected, we select the first entry )
		 ONE
		 #MAX
		 DUP
		 2PUTLAM
		 
		 FALSE ( do not quit the loop )
	       ;
	     DUP TWO #= case
	       ::
	         ( key[B]: edit )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] )

		 2PUTLAM
		 DROP
                 ( stack: [passwd-str]-[data-list] )
		 ( calculate the position of the entry to be edited )
		 1GETLAM
		 2GETLAM
		 #+
		 #1-
		 2DUP
                 ( stack: [passwd-str]-[data-list]-[pos index]-... )
		 (        ...-[data-list]-[pos index] )
		 ( get the entry )
		 NTHCOMPDROP

		 ( edit the entry )
                 ID editEntry
		 
                 ( stack: [passwd-str]-[data-list]-[pos index]-[changedEntry] )
		 ( store the entry )
		 SWAPROT
		 PUTLIST

		 ( recalculate the window )
		 1GETLAM
		 ID extractHeaders
		 2GETLAM 

		 ( restore the menu-line )
	         ID stdMenu

		 FALSE ( do not quit the loop )
	       ;
	     DUP THREE #= case
	       ::
	         ( key[C]: del )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] )
		 SWAPDROP

		 ( store selected pos )
		 DUP
		 2PUTLAM 

                 ( stack: [passwd-str]-[data-list]-[selected pos] )

		 ( calculate position to remove elem from )
		 1GETLAM
		 #+
		 #1-
		 REMOVEN{}

                 ( stack: [passwd-str]-[changed data-list] )

                 ( adjust the LAMs ) 
		 ( calc. the maximum for the start of the window )
		 DUP 
		 LENCOMP
		 DUP
		 SEVEN
		 #MAX
		 SIX
		 #-
                 ( adjust the start of the window )
		 1GETLAM
		 #MIN
		 1PUTLAM
		 ( adjust the selected position: )
                 ( stack: [passwd-str]-[data-list]-[length of data-list] )
		 2GETLAM
		 #MIN
		 2PUTLAM
		 
                 ( stack: [passwd-str]-[data-list] )
		 ( recalculate the window-data )
		 1GETLAM
		 ID extractHeaders
		 2GETLAM 
                 ( stack: [passwd-str]-[data-list]-[window-header]-[sel. pos] )
		 
		 FALSE ( do not quit the loop )
	       ;
	     DUP FIVE #= case
	       ::
	         ( key[E]: pass )
		 DROP ( we do not need the keycode any longer )
		 4ROLL
		 ID blankMenu
		 ID changePasswd
		 ID stdMenu
		 4UNROLL
		 FALSE ( do not quit the loop )
	       ;
	     DUP SIX #= case
	       ::
	         ( key[F]: quit'n'save )
		 3DROP ( drop the key, the sel-pos & the win-headers )
		 TRUE ( yes, save )
		 TRUE ( yes, exit loop )
	       ;
	     DUP ELEVEN #= case
	       ::
	         ( key[K]: cursor up ) 
		 DROPDUP ( we do not need the keycode any longer )
		 TWO
		 #<
		 ITE
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] )
		 ::
		   ( selecting up is scrolling the window up here )
		   1GETLAM
		   ONE
		   #>
		   IT
		   ::
		     ( we are not at the top, so we may go further up )
		     2PUTLAM ( store the selection ) 
		     DROP
		     1GETLAM 
		     #1-
		     DUP
		     1PUTLAM
		     ID extractHeaders
		     2GETLAM
		   ;
		 ;
		 ::
		   ( selecting up is decreasing the selector )
		   #1-
		 ;
		 FALSE ( do not quit the loop )
	       ;
	     DUP SEVENTEEN #= case
	       ::
	         ( key[Q]: cursor down )
		 DROPDUP ( we do not need the keycode any longer )
		 SIX
		 #>
		 ITE
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] )
		 ::
		   ( selecting down is scrolling the window down here )
		   3PICK
		   LENCOMP
		   1GETLAM 
		   #6+
		   #>
		   IT
		   ::
		     ( we are not at the bottom, so we may go further down )
		     2PUTLAM
		     DROP
		     1GETLAM
		     #1+
		     DUP
		     1PUTLAM
		     ID extractHeaders
		     2GETLAM
		   ;
		 ;
		 ::
		   ( selecting down is increasing the selector )
		   
                   ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	           (        ...-[selected pos] )
		   #1+
		   3PICK
		   LENCOMP
		   #MIN 
		 ;
		 FALSE ( do not quit the loop )
	       ;
	     DUP SIXTEEN #= case
	       ::
	         ( key[P]: cursor left --- scroll one page up )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] ) 
		 1GETLAM
		 DUP 
		 ONE
		 #=
		 ITE
		 ::
		   ( we may not scroll up any further --- therefore we )
		   ( do not need to change the win-header-list; we just )
		   ( select the first entry. )
		   2DROP
		   ONEONE
		   2PUTLAM
		 ;
		 ::
		   ( we may scroll up )
                   ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	           (        ...-[selected pos]-[window pos] ) 
		   ( recalculate the window-position )
		   SEVEN
		   #MAX
		   #6-
		   1PUTLAM
                   ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	           (        ...-[selected pos] ) 
		   ( recalculate the header-list )
		   2DROP
		   1GETLAM
		   ID extractHeaders
		   ( calculate the selected pos: the last selectable entry )
		   OVER
		   LENCOMP
		   SEVEN
		   #MIN
		   DUP 2PUTLAM
		 ;
		 FALSE ( do not quit the loop )
	       ;
	     DUP EIGHTEEN #= case
	       ::
	         ( key[R]: cursor right --- scroll one page down )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] ) 
		 1GETLAM
		 SEVEN
		 #+
		 4PICK
		 LENCOMP
		 #>
		 ITE
		 ::
		   ( we may not scroll down any further --- therefore we )
		   ( do not need to change the win-header-list; we just )
		   ( select the last entry. )
		   DROP
		   OVER
		   LENCOMP
		   SEVEN
		   #MIN
		   DUP
		   2PUTLAM
		 ;
		 ::
		   ( we may scroll down )
                   ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	           (        ...-[selected pos] ) 
		   ( recalculate the window-position )
		   2DROP
                   ( stack: [passwd-str]-[data-list] )
		   DUP
		   LENCOMP
		   SEVEN
		   #MAX
		   ( stack: [passwd-str]-[data-list]-[maximal window-pos + 6] )
		   1GETLAM
		   #6+
		   #6+
		   #MIN
		   #6-
                   ( stack: [passwd-str]-[data-list]-[new window-pos] ) 
		   DUP
		   1PUTLAM
		   ( recalculate the header-list )
		   ID extractHeaders
                   ( stack: [passwd-str]-[data-list]-[win-header list] )
		   ( calculate the selected pos: the first selectable entry )
		   OVER
		   LENCOMP
		   ONE
		   #MIN 
		   DUP 2PUTLAM
		 ;
		 FALSE ( do not quit the loop )
	       ;
	     DUP TWENTYFIVE #= case
	       ::
	         ( key[Enter]: view entry )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] ) 
		 ( get the data-list )
		 3PICK
		 ( calculate the entry-index )
		 OVER 
		 1GETLAM
		 #+
		 #1-
		 NTHCOMPDROP
		 ( show empty menu )
		 ID blankMenu
		 ( display the entry )
		 ZERO
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos]-[data-element]-[#0] ) 
		 BlankDA12
		 ID dispStrings 
		 
		 "  (Press any key)."
		 DISPROW7
		 WaitForKey
		 
		 ( clear up )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos]-[data-element]-[#0]-[key]-[mod] ) 
		 4DROP
		 
		 ( show standard menu )
		 ID stdMenu

		 FALSE ( do not quit the loop )
	       ;
	     DUP TWENTYEIGHT #= case
	       ::
	         ( key[Del]: turn HP off )
		 DROP ( we do not need the keycode any longer )
                 ( stack: [passwd-str]-[data-list]-[win-header list]-... )
	         (        ...-[selected pos] ) 
		 TurnOff 

		 ( ask for the password: )
                 4PICK
		 ID blankMenu
		 "- Turning HP on -"
		 ID getPasswd
		 EQUAL
		 
                 ITE
		 ::
		   ( the password's been ok... )
		   ID stdMenu
		   FALSE ( do not quit the loop )
		 ;
		 ::
		   ( the password's been wrong... )
		   
		   ( annoy the user )
		   BlankDA12
		   "(All changes unsaved)"
		   DISPROW1
		   "! Wrong Password !"
		   BIGDISPROW2
		   %3
		   dowait
		   
		   2DROP ( drop sel-pos & win-headers )
		   FALSE ( don't save )
		   TRUE ( exit loop )
		 ;
	       ;
	     DUP FORTYFIVE #= case
	       ::
	         ( key[Cancel]: abort without save )
		 3DROP ( drop the key, the sel-pos & the win-headers )
		 FALSE ( don't save )
		 TRUE  ( exit loop )
	       ;
	       ::
	         ( default )
		 DROP ( drop the key )
		 FALSE ( don't exit loop )
	       ;
	   ;
	 UNTIL
 
       ABND
    ;
;
